package com.xhkjedu.sclass.controller.classroom;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xhkjedu.vo.PageResult;
import com.xhkjedu.sclass.model.classroom.TClassroom;
import com.xhkjedu.sclass.model.classroom.TClassroomFile;
import com.xhkjedu.sclass.service.classroom.ClassroomService;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.utils.PageUtil;
import com.xhkjedu.vo.ResultVo;
import com.xhkjedu.sclass.vo.classroom.RoomVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("room")
@Slf4j
public class ClassroomController {

    @Resource
    private ClassroomService classroomService;

    /**
     * 获取课堂回顾列表
     * @Param [room]
     * @Author ywx
     * @Date 2020/6/15 10:57
     * @return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("list_class_directorid")
    public ResultVo listClassByDirectorId(@RequestBody TClassroom room) {
        try {
            Integer teacherid = room.getTeacherid();
            Integer page = room.getPage();
            Integer pageSize = room.getPageSize();
            N_Utils.validation(new Object[]{teacherid,"教师id",1,page,"显示页码",1,pageSize,"显示条数",1});
            PageHelper.startPage(page, pageSize);
            List<RoomVo> list = classroomService.listClassByDirectorId(room);
            PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(list));
            return new ResultVo(0,"获取课堂回顾列表成功",pageResult);
        } catch (Exception e) {
            log.error("获取课堂回顾列表失败：" + e.getMessage());
            return new ResultVo(1,"获取课堂回顾列表失败");
        }
    }

    /**
     * 课堂回顾详情
     * @Param [room]
     * @Author ywx
     * @Date 2020/6/26 9:36
     * @return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/report")
    public ResultVo report(@RequestBody TClassroom room) {
        N_Utils.validation(new Object[]{room.getRoomid(),"课堂id",1});
        return classroomService.getReport(room.getRoomid());
    }

    @PostMapping("/del_file")
    public ResultVo delFile(@RequestBody TClassroomFile file) {
        N_Utils.validation(new Object[]{file.getRfid(),"课堂图片id",1});
        classroomService.delFile(file.getRfid());
        return new ResultVo(0,"删除成功");
    }

    @PostMapping("/list_student")
    public ResultVo listStudent(@RequestBody TClassroom room) {
        N_Utils.validation(new Object[]{room.getClassid(),"班级id",1,room.getLsbid(),"教材关联id",2});
        List<Map> list = classroomService.listStudent(room.getClassid(), room.getLsbid());
        return new ResultVo(0,"获取班级学生列表成功",list);
    }
}
