package com.xhkjedu.sclass.mapper.classroom;

import com.xhkjedu.sclass.base.TkMapper;
import com.xhkjedu.sclass.model.classroom.TClassroom;
import com.xhkjedu.sclass.model.classroom.TClassroomFile;
import com.xhkjedu.sclass.vo.classroom.ReportEQuestionVo;
import com.xhkjedu.sclass.vo.classroom.ReportExamVo;
import com.xhkjedu.sclass.vo.classroom.ReportRoomVo;
import com.xhkjedu.sclass.vo.classroom.RoomVo;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

public interface ClassroomMapper extends TkMapper<TClassroom> {

    /**
     * 结束课堂
     * @Param [room]
     * @Author ywx
     * @Date 2020/6/15 9:07
     * @return java.lang.Integer
     **/
    Integer endClass(@Param("room") TClassroom room);

    /**
     * 获取教师指定章节下上过的课堂历史
     * @Param [room]
     * @Author ywx
     * @Date 2020/6/15 9:18
     * @return java.util.List<com.xhkjedu.vo.classroom.RoomVo>
     **/
    List<RoomVo> listClassByDirectorId(@Param("room") TClassroom room);

    /**
     *功能描述  批量更新班级实际签到学生数量
     * @author WN
     * @date 2020/8/14
     * @param  * @param roomidlist
     * @return void
     */
    void updateBatchRoomRealStunum(List<Long> roomidlist);


    /**
     * 根据班级id获取学生数量
     * @Param [classid]
     * @Author ywx
     * @Date 2020/6/15 9:08
     * @return java.lang.Integer
     **/
    Integer getStudentNumById(@Param("classid") Integer classid);

    /**
     * 课堂回顾详情
     * @Param [roomid]
     * @Author ywx
     * @Date 2020/6/26 9:36
     * @return com.xhkjedu.vo.classroom.ReportRoomVo
     **/
    ReportRoomVo getReport(Long roomid);

    /**
     * 课堂提问列表
     * @Param [roomid]
     * @Author ywx
     * @Date 2020/6/26 9:53
     * @return java.util.List<com.xhkjedu.vo.classroom.ReportQuestionVo>
     **/
    List<Map> listQuestionReport(Long roomid);

    /**
     * 课堂点名列表
     * @Param [roomid]
     * @Author ywx
     * @Date 2020/6/26 10:06
     * @return java.util.List<com.xhkjedu.vo.classroom.ReportStudentVo>
     **/
    List<Map> listCallReport(Long roomid);

    /**
     * 课堂抢答列表
     * @Param [roomid]
     * @Author ywx
     * @Date 2020/6/26 10:16
     * @return java.util.List<com.xhkjedu.vo.classroom.ReportStudentVo>
     **/
    List<Map> listAskReport(Long roomid);

    /**
     * 课堂截图列表
     * @Param [roomid]
     * @Author ywx
     * @Date 2020/6/26 10:24
     * @return java.util.List<com.xhkjedu.model.classroom.TClassroomFile>
     **/
    List<TClassroomFile> listFileReport(Long roomid);

    /**
     * 课堂考试列表
     * @Param [paperid,classid]
     * @Author ywx
     * @Date 2020/6/26 10:34
     * @return java.util.List<com.xhkjedu.vo.classroom.ReportEQuestionVo>
     **/
    List<ReportEQuestionVo> listExamReport(Long paperid, Integer classid);

    /**
     *功能描述  批量结束之前班级的课堂状态
     * @author WN
     * @date 2020/8/14
     * @param  * @param classidlist
     * @return void
     */
    void updateBatchRoomState(List<Integer> classidlist);

    /**
     * 获取课堂考试列表
     * @Param [roomid]
     * @Author ywx
     * @Date 2020/9/10 9:35
     * @return java.util.List<com.xhkjedu.vo.classroom.ReportExamVo>
     **/
    List<ReportExamVo> listPaper(Long roomid);

    //班级学生列表
    List<Map> listStudent(Integer classid, String lsbid);
}
