package com.xhkjedu.sclass.mapper.room;

import com.xhkjedu.sclass.base.TkMapper;
import com.xhkjedu.sclass.model.classroom.TClassroom;
import com.xhkjedu.sclass.vo.classroom.ObjectiveVo;
import com.xhkjedu.sclass.vo.classroom.RoomCorrectVo;
import com.xhkjedu.sclass.vo.classroom.RoomVo;
import com.xhkjedu.sclass.vo.classroom.SubjectiveVo;
import com.xhkjedu.sclass.vo.paper.CheckClassVo;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

public interface RoomMapper extends TkMapper<TClassroom> {
    /**
     * 保存课堂
     * @Param [r]
     * @Author ywx
     * @Date 2020/7/14 14:48
     * @return java.lang.Integer
     **/
    Integer save(@Param("r") TClassroom room);

    /**
     * 获取班级学生id集合
     * @Param [roomid]
     * @Author ywx
     * @Date 2020/7/14 17:01
     * @return java.util.List<java.lang.Integer>
     **/
    List<Integer> listStudentId(@Param("roomid") Long roomid);

    /**
     * 获取批阅信息
     * @Param [roomid]
     * @Author ywx
     * @Date 2020/9/9 15:38
     * @return com.xhkjedu.sclass.vo.classroom.RoomCorrectVo
     **/
    RoomCorrectVo listCorrect(Long roomid);

    /**
     * 获取提问批改学生列表
     * @Param [asktid]
     * @Author ywx
     * @Date 2020/9/9 16:39
     * @return com.xhkjedu.sclass.vo.paper.CheckClassVo
     **/
    CheckClassVo listStudentsByAskTid(Long asktid);

    /**
     * 获取考试批改学生列表
     * @Param [paperid]
     * @Author ywx
     * @Date 2020/9/9 17:12
     * @return com.xhkjedu.sclass.vo.paper.CheckClassVo
     **/
    CheckClassVo listStudentsByPaperId(Long paperid);

    /**
     * 获取课堂提问客观题详情
     * @Param [asktid]
     * @Author ywx
     * @Date 2020/9/10 11:12
     * @return com.xhkjedu.sclass.vo.classroom.ObjectiveVo
     **/
    ObjectiveVo callObjective(Long asktid, Integer realstunum);

    /**
     * 获取课堂考试客观题详情
     * @Param [pscaid]
     * @Author ywx
     * @Date 2020/9/10 11:21
     * @return com.xhkjedu.sclass.vo.classroom.ObjectiveVo
     **/
    ObjectiveVo examObjective(Long pscaid, Integer realstunum);

    /**
     * 获取课堂提问主客观题详情
     * @Param [asktid]
     * @Author ywx
     * @Date 2020/9/10 14:01
     * @return com.xhkjedu.sclass.vo.classroom.SubjectiveVo
     **/
    SubjectiveVo callSubjective(Long asktid, Integer realstunum);

    /**
     * 获取课堂考试主客观题详情
     * @Param [pscaid]
     * @Author ywx
     * @Date 2020/9/10 14:24
     * @return com.xhkjedu.sclass.vo.classroom.SubjectiveVo
     **/
    SubjectiveVo examSubjective(Long pscaid, Integer realstunum);

    /**
     * 课堂提问批阅
     * @Param [asksid, radiocorrect]
     * @Author ywx
     * @Date 2020/9/10 15:08
     * @return void
     **/
    void correctCall(Integer asksid, Integer radiocorrect);

    /**
     * 学生课堂回顾列表
     * @Param [room]
     * @Author ywx
     * @Date 2020/9/14 16:32
     * @return java.util.List<com.xhkjedu.sclass.vo.classroom.RoomVo>
     **/
    List<RoomVo> listStuClass(@Param("room") TClassroom room);

    /**
     * 学生课堂回顾详情
     * @Param [roomid, studentid]
     * @Author ywx
     * @Date 2020/9/14 16:57
     * @return com.xhkjedu.sclass.vo.classroom.ReportRoomStuVo
     **/
    Map getReportStu(Long roomid, Integer studentid);
}
