package com.xhkjedu.sclass.service.classroom;

import com.xhkjedu.sclass.mapper.classroom.ClassroomFileMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomSignMapper;
import com.xhkjedu.sclass.mapper.room.RoomPaperFileMapper;
import com.xhkjedu.sclass.model.classroom.TClassroom;
import com.xhkjedu.sclass.model.classroom.TClassroomFile;
import com.xhkjedu.sclass.vo.classroom.ReportEQuestionVo;
import com.xhkjedu.sclass.vo.classroom.ReportExamVo;
import com.xhkjedu.sclass.vo.classroom.ReportRoomVo;
import com.xhkjedu.sclass.vo.classroom.RoomVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class ClassroomService {
    @Resource
    private ClassroomMapper classroomMapper;
    @Resource
    private ClassroomSignMapper classroomSignMapper;
    @Resource
    private RoomPaperFileMapper roomPaperFileMapper;
    @Resource
    private ClassroomFileMapper classroomFileMapper;

    /**
     * 获取教师指定章节下上过的课堂历史
     * @Param [room]
     * @Author ywx
     * @Date 2020/6/15 9:19
     * @return java.util.List<com.xhkjedu.sclass.vo.classroom.RoomVo>
     **/
    public List<RoomVo> listClassByDirectorId(TClassroom room) {
        return classroomMapper.listClassByDirectorId(room);
    }

    /**
     * 课堂回顾详情
     * @Param [roomid]
     * @Author ywx
     * @Date 2020/6/26 9:36
     * @return com.xhkjedu.sclass.vo.ResultVo
     **/
    public ResultVo getReport(Long roomid) {
        ReportRoomVo room = classroomMapper.getReport(roomid);
        Integer realstunum = room.getRealstunum();
        //课堂提问列表
        List<Map> questions = classroomMapper.listQuestionReport(roomid);
        room.setQuestions(questions);

        //课堂考试列表
        List<ReportExamVo> exams = classroomMapper.listPaper(roomid);
        for (ReportExamVo exam : exams) {
            realstunum = exam.getYtjnum();
            List<String> paperfiles = roomPaperFileMapper.listFilepath(exam.getPaperid());
            exam.setPaperfiles(paperfiles);

            List<ReportEQuestionVo> equestions = classroomMapper.listExamReport(exam.getPaperid(),exam.getClassid());
            for (ReportEQuestionVo equestion : equestions) {
                Double correctrate = N_Utils.getDoubleDivideAndMulitiply(equestion.getCorrectrate(),realstunum);
                equestion.setCorrectrate(correctrate);
                equestion.setCommitrate(N_Utils.getDoubleDivideAndMulitiply(equestion.getCommitrate(),realstunum));
            }
            exam.setQuestions(equestions);
        }
        room.setExams(exams);

        //课堂点名列表
        List<Map> calls = classroomMapper.listCallReport(roomid);
        room.setCalls(calls);

        //课堂抢答列表
        List<Map> asks = classroomMapper.listAskReport(roomid);
        room.setAsks(asks);

        //课堂截图列表
        List<TClassroomFile> files = classroomMapper.listFileReport(roomid);
        room.setFiles(files);

        //课堂学生列表
        List<Map> students = classroomSignMapper.listStudent(roomid);
        room.setStudents(students);
        return new ResultVo(0, "获取课堂回顾详情成功",room);
    }

    //删除课堂图片
    public void delFile(Integer rfid) {
        classroomFileMapper.deleteByPrimaryKey(rfid);
    }

    //班级学生列表
    public List<Map> listStudent(Integer classid, String lsbid) {
        return classroomMapper.listStudent(classid, lsbid);
    }
}
