package com.xhkjedu.sclass.service.classroom;

import com.xhkjedu.sclass.mapper.classroom.ClassroomActivestudentMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomMapper;
import com.xhkjedu.sclass.mapper.room.RoomAskstudentMapper;
import com.xhkjedu.sclass.mapper.room.RoomAskteacherMapper;
import com.xhkjedu.sclass.mapper.room.RoomFileMapper;
import com.xhkjedu.sclass.mapper.room.RoomMapper;
import com.xhkjedu.sclass.mapper.room.RoomPaperFileMapper;
import com.xhkjedu.sclass.mapper.room.RoomSignMapper;
import com.xhkjedu.sclass.model.classroom.TClassroom;
import com.xhkjedu.sclass.model.classroom.TClassroomAskstudent;
import com.xhkjedu.sclass.model.classroom.TClassroomAskteacher;
import com.xhkjedu.sclass.model.classroom.TClassroomFile;
import com.xhkjedu.sclass.model.classroom.TClassroomPaperFile;
import com.xhkjedu.sclass.model.classroom.TClassroomSign;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import com.xhkjedu.sclass.vo.classroom.ObjectiveVo;
import com.xhkjedu.sclass.vo.classroom.RoomCorrectCallVo;
import com.xhkjedu.sclass.vo.classroom.RoomCorrectExamVo;
import com.xhkjedu.sclass.vo.classroom.RoomCorrectVo;
import com.xhkjedu.sclass.vo.classroom.RoomVo;
import com.xhkjedu.sclass.vo.classroom.SubjectiveVo;
import com.xhkjedu.sclass.vo.paper.CheckClassVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class RoomService {
    @Resource
    private ClassroomMapper classroomMapper;
    @Resource
    private RoomAskteacherMapper roomAskteacherMapper;
    @Resource
    private ClassroomActivestudentMapper classroomActivestudentMapper;
    @Resource
    private RoomAskstudentMapper roomAskstudentMapper;
    @Resource
    private RoomSignMapper roomSignMapper;
    @Resource
    private RoomFileMapper roomFileMapper;
    @Resource
    private RoomMapper roomMapper;
    @Resource
    private RoomPaperFileMapper roomPaperFileMapper;

    /**
     * 开始课堂
     * @Param [room]
     * @Author ywx
     * @Date 2020/7/14 17:17
     * @return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public boolean startClass(TClassroom room) {
        boolean rtn = false;//课堂是否重复请求
        try {
            room.setRoomstate(1);
            Integer num = classroomMapper.getStudentNumById(room.getClassid());//根据班级id获取学生数量
            room.setClassnum(num);
            roomMapper.save(room);
            classroomActivestudentMapper.saveStuActive(room.getRoomid(),room.getClassid(),room.getSchoolid());
        } catch (Exception e) {
            if (e instanceof DuplicateKeyException) {
                rtn = true;
                log.error("接口重复请求，课堂id：" + room.getRoomid()+",课堂名称："+room.getRoomname());
            } else {
                log.error("开始课堂失败：" + e.getMessage());
            }
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return rtn;
    }

    /**
     * 结束课堂
     * @Param [room]
     * @Author ywx
     * @Date 2020/7/14 17:17
     * @return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public void stopClass(TClassroom room) {
        try {
            Long roomid = room.getRoomid();
            if (N_Utils.isEmptyInteger(room.getEndtime())) {//异常结束：结束时间=开始时间+45分钟
                TClassroom tClassroom = roomMapper.selectByPrimaryKey(roomid);
                room.setEndtime(tClassroom.getCreatetime() + 60 * 45);
            }
            classroomMapper.endClass(room);//结束课堂
            classroomActivestudentMapper.updateActiveRank(roomid);//更新活跃值排名
        } catch (Exception e) {
            log.error("结束课堂失败：" + e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }

    /**
     * 保存学生签到信息
     * @Param [sign]
     * @Author ywx
     * @Date 2020/7/14 16:32
     * @return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveStudentSign(TClassroomSign sign) {
        try {
            Long roomid = sign.getRoomid();
            Integer userid = sign.getUserid();
            List<Integer> studentids = roomMapper.listStudentId(roomid);//获取班级学生id集合
            if (N_Utils.numInList(userid, studentids)) {//签到学生是本课堂所在班级学生，添加签到信息
                roomSignMapper.save(sign);
                //如果一个学生退出课堂再进来数据就不对了，最后计算
                //classroomMapper.updateRealStuNum(sign.getRoomid());//更新班级实际签到学生数量
                classroomActivestudentMapper.updateActiveNum(roomid,userid);//更新学生活跃值
            }
        } catch (Exception e) {
            log.error("保存学生签到信息失败：" + e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }


    /**
     *功能描述 最后更新课堂名称
     * @author WN
     * @date 2020/8/14
     * @param  * @param roomlist
     * @return void
     */
    public void updateBatchRoomRealStunum(List<Long> roomlist){
        classroomMapper.updateBatchRoomRealStunum(roomlist);
    }

    /**
     * 互动发起
     * @Param [teacher]
     * @Author ywx
     * @Date 2020/6/15 9:58
     * @return void
     **/
    public void saveWork(TClassroomAskteacher teacher) {
        try {
            if (teacher.getPaperid() == null){
                teacher.setPaperid(0L);
            }
            if (teacher.getRadionum() == null){
                teacher.setRadionum(0);
            }
            roomAskteacherMapper.save(teacher);
        } catch (Exception e) {
            log.error("互动发起失败：" + e.getMessage());
        }
    }

    /**
     * 保存学生作答情况
     * @Param [student]
     * @Author ywx
     * @Date 2020/7/14 16:34
     * @return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveStuAnswer(TClassroomAskstudent student) {
        try {
            if (student.getStudentradio() == null){
                student.setStudentradio("");
            }
            if (student.getStudentpic() == null){
                student.setStudentpic("");
            }
            if (student.getRadiocorrect() == null){
                student.setRadiocorrect(0);
            }
            if (student.getStudentselected() == null){
                student.setStudentselected(0);
            }
            roomAskstudentMapper.save(student);
            //保存成功更新学生活跃值
            classroomActivestudentMapper.updateActiveNum(student.getRoomid(),student.getStudentid());
        } catch (Exception e) {
            log.error("保存学生作答情况失败：" + e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }

    /**
     * 保存课堂图片
     * @Param [file]
     * @Author ywx
     * @Date 2020/7/14 16:42
     * @return void
     **/
    public void saveFile(TClassroomFile file) {
        try {
            roomFileMapper.save(file);
        } catch (Exception e) {
            log.error("批量保存课堂图片失败："+e.getMessage());
        }
    }

    /**
     *功能描述 批量结束班级之前的课堂
     * @author WN
     * @date 2020/8/14
     * @param  * @param classidlist
     * @return void
     */
    public void updateBatchRoomState(List<Integer> classidlist){
        classroomMapper.updateBatchRoomState(classidlist);
    }

    /**
     * 获取批阅信息
     * @Param [roomid]
     * @Author ywx
     * @Date 2020/9/9 15:37
     * @return com.xhkjedu.sclass.vo.classroom.RoomCorrectVo
     **/
    public RoomCorrectVo listCorrect(Long roomid) {
        RoomCorrectVo correct = roomMapper.listCorrect(roomid);
        List<RoomCorrectCallVo> calls = correct.getCalls();
        if (N_Utils.isListNotEmpty(calls)){
            for (RoomCorrectCallVo call : calls) {
                call.setCorrectrate(N_Utils.getIntegerDivideAndMulitiply(call.getCorrectnum(),call.getStunum()));
            }
        }
        List<RoomCorrectExamVo> exams = correct.getExams();
        if (N_Utils.isListNotEmpty(exams)){
            for (RoomCorrectExamVo exam : exams) {
                exam.setCorrectrate(N_Utils.getIntegerDivideAndMulitiply(exam.getCorrectnum(),exam.getStunum()));
            }
        }
        return correct;
    }

    /**
     * 获取提问批改学生列表
     * @Param [asktid]
     * @Author ywx
     * @Date 2020/9/9 16:38
     * @return com.xhkjedu.sclass.vo.paper.CheckClassVo
     **/
    public CheckClassVo listStudentsByAskTid(Long asktid) {
        CheckClassVo classVo = roomMapper.listStudentsByAskTid(asktid);
        return getCheckClassVo(classVo);
    }

    /**
     * 计算未提交数、批阅进度
     * @Param [classVo]
     * @Author ywx
     * @Date 2020/9/10 8:56
     * @return com.xhkjedu.sclass.vo.paper.CheckClassVo
     **/
    private CheckClassVo getCheckClassVo(CheckClassVo classVo) {
        Integer ytjnum = classVo.getYtjnum();
        classVo.setWtjnum(classVo.getClassnum() - ytjnum);
        if (!N_Utils.isEmptyInteger(ytjnum)) {
            double scale = N_Utils.getIntegerDivideAndMulitiply(classVo.getYpynum(), ytjnum);
            classVo.setScale(scale);
        } else {
            classVo.setScale(0);
        }
        return classVo;
    }

    /**
     * 获取考试批改学生列表
     * @Param [paperid]
     * @Author ywx
     * @Date 2020/9/9 17:12
     * @return com.xhkjedu.sclass.vo.paper.CheckClassVo
     **/
    public CheckClassVo listStudentsByPaperId(Long paperid) {
        CheckClassVo classVo = roomMapper.listStudentsByPaperId(paperid);
        List<TClassroomPaperFile> files = roomPaperFileMapper.listObjsByPaperid(paperid , 3);
        classVo.setFiles(files);
        return getCheckClassVo(classVo);
    }

    /**
     * 获取课堂提问客观题详情
     * @Param [asktid]
     * @Author ywx
     * @Date 2020/9/10 11:12
     * @return com.xhkjedu.sclass.vo.classroom.ObjectiveVo
     **/
    public ObjectiveVo callObjective(Long asktid, Integer realstunum) {
        return roomMapper.callObjective(asktid, realstunum);
    }

    /**
     * 获取课堂考试客观题详情
     * @Param [pscaid]
     * @Author ywx
     * @Date 2020/9/10 11:21
     * @return com.xhkjedu.sclass.vo.classroom.ObjectiveVo
     **/
    public ObjectiveVo examObjective(Long pscaid, Integer realstunum) {
        return roomMapper.examObjective(pscaid, realstunum);
    }

    /**
     * 获取课堂提问主客观题详情
     * @Param [asktid]
     * @Author ywx
     * @Date 2020/9/10 14:01
     * @return com.xhkjedu.sclass.vo.classroom.SubjectiveVo
     **/
    public SubjectiveVo callSubjective(Long asktid, Integer realstunum) {
        return roomMapper.callSubjective(asktid, realstunum);
    }

    /**
     * 获取课堂考试主客观题详情
     * @Param [pscaid]
     * @Author ywx
     * @Date 2020/9/10 14:25
     * @return com.xhkjedu.sclass.vo.classroom.SubjectiveVo
     **/
    public SubjectiveVo examSubjective(Long pscaid, Integer realstunum) {
        return roomMapper.examSubjective(pscaid, realstunum);
    }

    /**
     * 课堂提问批阅
     * @Param [asksid, radiocorrect]
     * @Author ywx
     * @Date 2020/9/10 15:08
     * @return void
     **/
    public void correctCall(Integer asksid, Integer radiocorrect) {
        roomMapper.correctCall(asksid,radiocorrect);
    }

    /**
     * 修改学生作答记录（教师添加批注）
     * @Param [asksid, studentpic]
     * @Author ywx
     * @Date 2020/9/11 15:08
     * @return void
     **/
    public void updateStudentpic(Integer asksid, String studentpic) {
        roomAskstudentMapper.updateStudentpic(asksid, studentpic);
    }

    /**
     * 学生课堂回顾列表
     * @Param [room]
     * @Author ywx
     * @Date 2020/9/14 16:32
     * @return java.util.List<com.xhkjedu.sclass.vo.classroom.RoomVo>
     **/
    public List<RoomVo> listStuClass(TClassroom room) {
        return roomMapper.listStuClass(room);
    }

    public ResultVo getReportStu(Long roomid, Integer studentid) {
        Map room = roomMapper.getReportStu(roomid, studentid);
        return new ResultVo(0, "获取学生课堂回顾详情成功",room);
    }
}
